;; disc loader 

plus equ &0040
playcity equ &0041

loader_load_addr equ &9000
km_wait_char equ &bb06
txt_output equ &bb5a
scr_initialise equ &bbff
scr_set_mode equ &bc0e
scr_set_border equ &bc38
cas_in_open equ &bc77
cas_in_direct equ &bc83
cas_in_close equ &bc7a
scr_set_ink equ &bc32
kl_del_frame_fly equ &bcdd
kl_time_please equ &bd0d

org loader_load_addr

exe_start:
ld bc,end_loader
or a
sbc hl,bc
bit 7,h
jr z,exe_start2

ld hl,memory_error_msg
call display_msg
call km_wait_char
rst 0

exe_start2:
call scr_initialise

ld e,14
ld hl,crtc_vals
ld bc,&bc00
set_crtc_vals:
out (c),c
inc b
ld a,(hl)
out (c),a
inc hl
dec b
inc c
dec e
jr nz,set_crtc_vals

;; display message
ld hl,gs_col_msg
call display_msg

;; detect playcity
call check_playcity
ld a,0
jr nz,noplaycity
ld hl,playcity_msg
call display_msg
ld a,1
noplaycity:
ld (playcity),a

call detect_plus
ld a,0
jr nz,noplus
ld hl,plus_msg
call display_msg

call plus_Unlock

ld a,1
noplus:
ld (plus),a

;; small pause
ld b,5
pause:
ld hl,0
pause2:
dec hl
ld a,h
or l
jr nz,pause2
djnz pause

ld a,(&8000)
or a
jr z,normal_screen

;; Bruno's title screen
ld a,1
call scr_set_mode

ld hl,title_colours_bruno
call set_palette

;; randomly load screen made by Bruno
ld hl,end_filename-1
ld (hl),'0'-1
ld de,&c000
call load_file
ld hl,end_filename-1
ld (hl),'1'

jr done_load_scr

normal_screen:
xor a
call scr_set_mode

ld a,(plus)
or a
jr z,no_plus_pal


ld      hl,&b7f9
call kl_del_frame_fly

ld hl,loadscr_plus_palette
call setup_plus_palette

jr do_load_scr

no_plus_pal:
ld hl,title_colours
call set_palette

do_load_scr:

ld de,&c000
call load_file

done_load_scr:

defb &11
include "reloc.lod_s.asm"
call load_file


di
ld a,(plus)
or a
jr z,no_fade

ld bc,&7fb8
out (c),c

ld de,&6400
call init_fade_out

ld b,16
fadeit:
push bc
ld b,&f5
fadeit2:
in a,(c)
rra
jr nc,fadeit2
fadeit2b:
in a,(c)
rra
jr c,fadeit2b

call update_fade_out

call update_fade_cols

pop bc
djnz fadeit
ld bc,&7fa0
out (c),c

call setup_plus_sprites
no_fade:

ld a,(plus)
defb &c3
include "reloc.lod_x.asm"



display_msg:
ld a,(hl)
and &7f
bit 7,(hl)
jp nz,txt_output
inc hl
call txt_output
jr display_msg

plus_msg:
defb 31,(20-(13/2)),13
defb "PLUS DETECTE","D"+&80

playcity_msg:
defb 31,(20-(17/2)),15
defb "PLAYCITY DETECTE","D"+&80

gs_col_msg:
defb 4,1
defb 29,0,0
defb 28,0,0,0
defb 28,1,26,26
defb 28,2,15,15
defb 28,3,25,25
defb 14,0
defb 15,1
defb 31,(20-(12/2)),6
defb "HEXAVIRUS v0"
defb &80

memory_error_msg:
defb 4,1
defb 29,0,0
defb 28,0,0,0
defb 28,1,26,26
defb 28,2,15,15
defb 28,3,25,25
defb 14,0
defb 15,1
defb 31,1,12
defb "Top address of free ram is too low.",13,10
defb "Please disable some expansion ROMs.",13,10
defb 13,10
defb "Press space to rese","t"+&80

;; detect plus
detect_plus:
  ld    bc,&f782
  out    (c),c
  dec    b
  ld    a,&f
  out    (c),a
  inc    b
  out    (c),c
  dec    b
  in    c,(c)
  cp    c 
  ret    

;; colours for screen
title_colours:
include "gfx/bin/loadscrfirmpal.asm"
defb 0 ;; border

crtc_vals:
defb &3f, 40, 46, &8e, &26, &00, 25, 30, &00, &07, &00,&00,&30,&00


include "pcdetect.asm"
include "fade.asm"
include "setup_plus.asm"



plus_Unlock:
ld hl,plus_Sequence
ld bc,&bc00+15
pu1:
ld a,(hl)
out (c),a
inc hl
dec c
jr nz,pu1
ld a,&cd
out (c),a
ld a,&ee
out (c),a
ret

plus_Sequence:
defb &ff, &0
defb &ff,&77,&b3,&51,&a8,&d4,&62,&39,&9c,&46,&2b,&15,&8a


set_palette:
ld b,16
xor a

set_colours:
push af
push bc
ld c,(hl)
inc hl
ld b,c
push hl
call scr_set_ink
pop hl
pop bc
pop af
inc a
djnz set_colours
ld c,(hl)
inc hl
ld b,c
call scr_set_border
ret

title_colours_bruno:
defb 24,9,18,26
defs 16-4
defb 9

load_file:
ld hl,end_filename-1
inc (hl)
ld hl,filename
ld b,end_filename-filename
push de
ld de,&c000
call cas_in_open
pop hl
call cas_in_direct
jp cas_in_close

filename:
defb "hexvirus.bi0"
end_filename:

end_loader:

end exe_start
