
ld sp,stack
ld bc,&7f00+%10001100	;; disable upper and lower rom and set mode 0
out (c),c

;; load screen
ld a,1				;; file 1
ld hl,&c000			;; to screen 
call load_a_file

ld bc,&7fb8
out (c),c
ld hl,plus_title_colours
ld de,&6400
ld bc,17*2
ldir
ld bc,&7fa0
out (c),c


;; waits for 3 seconds OR a key/button press
;; delay here or press a key
ld bc,3*50
wl0:
push bc
wl1:
;; wait vsync start
ld b,&f5
in a,(c)
rra
jr nc,wl1
;; some interrupts to skip vsync
wl4:
in a,(c)
rra
jr c,wl4
call read_matrix
pop bc
;; space
ld a,(matrix_buffer+5)
bit 7,a
jr nz,wl2
;; fire 1,2 on pad 2
ld a,(matrix_buffer+6)
and %00110000
jr nz,wl2
;; fire 1,2 on pad 1
ld a,(matrix_buffer+9)
and %00110000
jr nz,wl2
jr wl3

wl2:
ld bc,1 ;; quit early by making count 1
wl3:
dec bc
ld a,b
or c
jr nz,wl0


ld bc,&7fb8
out (c),c

ld de,&6400
call init_fade_out

ld b,16
fadeit:
push bc
ld b,&f5
fadeit2:
in a,(c)
rra
jr nc,fadeit2
fadeit2b:
in a,(c)
rra
jr c,fadeit2b

call update_fade_out

call update_fade_cols

pop bc
djnz fadeit
ld bc,&7fa0
out (c),c

call setup_plus_sprites

ld hl,load_main
ld de,&4000
ld bc,end_load_main-load_main
ldir
jp &4000

load_main:
defb &31
include "hexavirus_s.asm"

;; load code
ld a,2
defb &21
include "hexavirus_s.asm"

push af
push bc
ld bc,&7f00+%10000000	;; enable upper and lower rom and set mode 0
out (c),c
pop bc
pop af
call &0003				;; execute "load_file"
ld bc,&7f00+%10001100	;; disable upper and lower rom and set mode 0
out (c),c
;; indicate it's plus
ld a,1
defb &c3
include "hexavirus_x.asm"
end_load_main:

plus_title_colours:
include "gfx/bin/loadscrpluspal.asm"
defw 0

;; this code must be in the central ram (&4000-&bfff) area to work correctly because
;; it enables lower and upper rom.
load_a_file:
;;di						;; disable interrupts
push af
push bc
ld bc,&7f00+%10000000	;; enable upper and lower rom and set mode 0
out (c),c
pop bc
pop af
call &0003				;; execute "load_file"
ld bc,&7f00+%10001100	;; disable upper and lower rom and set mode 0
out (c),c
;;ei						;; enable interrupts
ret


read_matrix:
ld hl,matrix_buffer        ; buffer to store matrix data

ld bc,&f40e                ; write PSG register index (14) to PPI port A (databus to PSG)
out (c),c

ld b,&f6
in a,(c)
and &30
ld c,a

or &C0                     ; bit 7=bit 6=1 (PSG operation: write register index)
out (c),a                  ; set PSG operation -> select PSG register 14

;; at this point PSG will have register 14 selected.
;; any read/write operation to the PSG will act on this register.

out (c),c                  ; bit 7=bit 6=0 (PSG operation: inactive)

inc b
ld a,&92
out (c),a                  ; write PPI control: port A: input, port B: input, port C upper: output
                           ; port C lower: output
push bc
set 6,c                    ; bit 7=0, bit 6=1 (PSG operation: read register data)

scan_key:
ld b,&f6 
out (c),c                 ;set matrix line & set PSG operation

ld b,&f4                  ;PPI port A (databus to/from PSG)
in a,(c)                  ;get matrix line data from PSG register 14

cpl                       ;invert data: 1->0, 0->1
                          ;if a key/joystick button is pressed bit will be "1"
                          ;keys that are not pressed will be "0"

ld (hl),a                 ;write line data to buffer
inc hl                    ;update position in buffer
inc c                     ;update line

ld a,c
and &0f
cp &0a                    ;scanned all rows?
jr nz,scan_key            ;no loop and get next row

;; scanned all rows
pop bc

ld a,&82                  ;write PPI Control: Port A: Output, Port B: Input, Port C upper: output, Port C lower: output.
out (c),a

dec b
out (c),c                 ;set PSG operation: bit7=0, bit 6=0 (PSG operation: inactive)
ret

;; This buffer has one byte per keyboard line. 
;; Each byte defines a single keyboard line, which defines 
;; the state for up to 8 keys.
;;
;; A bit in a byte will be '1' if the corresponding key 
;; is pressed, '0' if the key is not pressed.
matrix_buffer:
defs 10

include "fade.asm"
include "setup_plus.asm"

defs 256
stack:

