; ---------------------------------------------------------------------------
;*** start of arkos tracker player

;    arkos tracker player v1.01 - cpc & msx version.
;    21/09/09

;    code by targhan/arkos.
;    psg registers sendings based on madram/overlander's optimisation trick.
;    restoring interruption status snippet by grim/arkos.

;    v1.01 additions
;    ---------------
;    - small (but not useless!) optimisations by grim/arkos at the ply_track1_waitcounter / ply_track2_waitcounter / ply_track3_waitcounter labels.
;    - optimisation of the r13 management by grim/arkos.

;    this player can adapt to the following machines =
;    amstrad cpc and msx.
;    output codes are specific, as well as the frequency tables.

;    this player modifies all these registers = hl, de, bc, af, hl', de', bc', af', ix, iy.
;    the stack is used in conventionnal manners (call, ret, push, pop) so integration with any of your code should be seamless.
;    the player does not modifies the interruption state, unless you use the ply_systemfriendly flag, which will cut the
;    interruptions at the beginning, and will restore them only if needed.

;    basically, there are three kind of players.

;    asm
;    ---
;    used in your asm productions. you call the player by yourself, you don't care if all the registers are modified.

;    set ply_systemfriendly and ply_usefirmwareinterruptions to 0.

;    in assembler =
;    ld de,musicaddress
;    call player / ply_init        to initialise the player with your song.
;    then
;    call player + 3 / ply_play    whenever you want to play/continue the song.
;    call player + 6 / ply_stop    to stop the song.

;    fades in/out
;    ------------
;    the player allows the volume to be modified. it provides the interface, but you'll have to set the volume by yourself.
;    set ply_usefades to 1.
;    in assembler =
;    ld e,volume (0=full volume, 16 or more=no volume)
;    call ply_setfadevalue

;    sound effects
;    -------------
;    the player manages sound effects. they must be defined in another song, generated as a "sfx music" in the arkos tracker.
;    set the ply_usesoundeffects to 1.

;    in assembler =
;    ld de,sfxmusicaddress
;    call ply_sfx_init        to initialise the sfx song.

;    then initialise and play the "music" song normally.

;    to play a sound effect =
;    a = no channel (0,1,2)
;    l = sfx number (>0)
;    h = volume (0...f)
;    e = note (0...143)
;    d = speed (0 = as original, 1...255 = new speed (1 is the fastest))
;    bc = inverted pitch (-#ffff -> ffff). 0 is no pitch. the higher the pitch, the lower the sound.
;    call ply_sfx_play
;    to stop a sound effect =
;    ld e,no channel (0,1,2)
;    call ply_sfx_stop
;    to stop the sound effects on all the channels =
;    call ply_sfx_stopall

;    for more information, check the manual.

;    any question, complaint, a need to reward? write to contact@julien-nevo.com
; ---------------------------------------------------------------------------
    list
ply_usesoundeffects equ 0       ; set to 1 if you want to use sound effects in your player. both cpu and memory consuming.

ply_usefades        equ 1       ; set to 1 to allow fades in/out. a little cpu and memory consuming.
                                ; ply_setfadevalue becomes available.

ply_retrigvalue     equ $fe     ; value used to trigger the retrig of register 13. 
                                ; $fe corresponds to cp xx. do not change it!

ply_speedcpt	equ 0
ply_heightcpt 	equ ply_speedcpt+1
ply_digidrum	equ ply_heightcpt+1
ply_linker_pt equ ply_digidrum+1
ply_specialtrack_pt equ ply_linker_pt+2
ply_specialtrack_waitcounter equ ply_specialtrack_pt+2



;; for each track
ply_track_waitcounter equ 0
ply_track_pt equ ply_track_waitcounter+1
ply_track_volume equ ply_track_pt+1
ply_track_transposition equ ply_track_volume+1
ply_track_pitch equ ply_track_transposition+1
ply_track_pitchadd equ ply_track_pitch+2
ply_track_instrument equ ply_track_pitchadd+2
ply_track_instrument_speed equ ply_track_instrument+2
ply_track_instrument_speed_cpt equ ply_track_instrument_speed+1
ply_track_saveptinstrument equ ply_track_instrument_speed_cpt+1
ply_track_instrumentstablept equ ply_track_saveptinstrument+2
ply_track_size equ ply_track_saveptinstrument+2

track1_offs equ 0
track2_offs equ track1_offs+ply_track_size
track3_offs equ track2_offs+ply_track_size

track_waitcounter equ 0

;; IX = player data

; ---------------------------------------------------------------------------
ply_play
    xor  a
    ld   (ix+ply_digidrum),a       ; reset the digidrum flag.

; manage speed. if speed counter is over, we have to read the pattern further.
    ld   a,(ix+ply_speedcpt)
    dec  a
    jp   nz,ply_speedend

; moving forward in the pattern. test if it is not over.

    ld   a,(ix+ply_heightcpt)
    dec  a
    jr   nz,ply_heightend

; pattern over. we have to read the linker.

; get the transpositions, if they have changed, or detect the song ending!
	ld l,(ix+ply_linker_pt+0)
	ld h,(ix+ply_linker_pt+1)
    ld   a,(hl)
    inc  hl
    rra
    jr   nc,ply_songnotover
    
    ; song over! we read the address of the loop point.
    ld   a,(hl)
    inc  hl
    ld   h,(hl)
    ld   l,a
    ld   a,(hl)                 ; we know the song won't restart now, so we can skip the first bit.
    inc  hl
    rra
ply_songnotover
    rra
    jr   nc,ply_nonewtransposition1
	ld a,(hl)
	ld (ix+ply_track_transposition+track1_offs),a
	inc hl

ply_nonewtransposition1
    rra
    jr   nc,ply_nonewtransposition2
	ld a,(hl)
	ld (ix+ply_track_transposition+track2_offs),a
	inc hl

ply_nonewtransposition2
    rra
    jr   nc,ply_nonewtransposition3
	ld a,(hl)
	ld (ix+ply_track_transposition+track3_offs),a
	inc hl

ply_nonewtransposition3

    ; get the tracks addresses.
	ld a,(hl)
	ld (ix+ply_track_pt+0+track1_offs),a
	inc hl
	ld a,(hl)
	ld (ix+ply_track_pt+1+track1_offs),a
	inc hl
	ld a,(hl)
	ld (ix+ply_track_pt+0+track2_offs),a
	inc hl
	ld a,(hl)
	ld (ix+ply_track_pt+1+track2_offs),a
	inc hl
	ld a,(hl)
	ld (ix+ply_track_pt+0+track3_offs),a
	inc hl
	ld a,(hl)
	ld (ix+ply_track_pt+1+track3_offs),a
	inc hl

    ; get the special track address, if it has changed.
    rra
    jr   nc,ply_nonewheight
	ld a,(hl)
	ld (ix+ply_height),a
	inc hl
ply_nonewheight

    rra
    jr   nc,ply_nonewspecialtrack
ply_newspecialtrack
    ld   a,(hl)
    ld   (ix+ply_specialtrack_pt + 0),a
    inc  hl
    ld   a,(hl)
    ld   (ix+ply_specialtrack_pt + 1),a
    inc  hl
	
ply_nonewspecialtrack
    ld   (ix+ply_linker_pt + 0),l
    ld   (ix+ply_linker_pt + 1),h

    ; reset the specialtrack/tracks line counter.
    ; we can't rely on the song data, because the pattern height is not related to the tracks height.
    ld   a,1
    ld   (ix+ply_specialtrack_waitcounter),a
    ld   (ix+ply_track_waitcounter + track1_offs),a
    ld   (ix+ply_track_waitcounter + track2_offs),a
    ld   (ix+ply_track_waitcounter + track3_offs),a

	ld a,(ix+ply_height)
ply_heightend
    ld   (ix+ply_heightcpt),a
	
;-------------------------------
; read the special track/tracks.
;-------------------------------

; read the special track.
	ld a,(ix+ply_specialtrack_waitcounter)
    dec  a
    jr   nz,ply_specialtrack_wait

	ld l,(ix+ply_specialtrack_pt+0)
	ld h,(ix+ply_specialtrack_pt+1)
    ld   a,(hl)
    inc  hl
    srl  a                      ; data (1) or wait (0)?
    jr   nc,ply_specialtrack_newwait    ; if wait, a contains the wait value.
    ; data. effect type?
    srl  a                              ; speed (0) or digidrum (1)?
    ; first, we don't test the effect type, but only the escape code (=0)
    jr   nz,ply_specialtrack_noescapecode
    ld   a,(hl)
    inc  hl
    
ply_specialtrack_noescapecode
    ; now, we test the effect type, since the carry didn't change.
    jr   nc,ply_specialtrack_speed
    ld   (ix+ply_digidrum),a
    jr   ply_pt_specialtrack_enddata

ply_specialtrack_speed
    ld   (ix+ply_speed),a
ply_pt_specialtrack_enddata
    ld   a,1
ply_specialtrack_newwait
    ld   (ix+ply_specialtrack_pt+0),l
    ld   (ix+ply_specialtrack_pt+1),h
ply_specialtrack_wait
    ld   (ix+ply_specialtrack_waitcounter),a

	
	
	
	ld ix,track1
	call read_track
	ld ix,track2
	call read_track
	ld ix,track3
	call read_track

    ld iy,ply_psgregistersarray + 4
	ld ix,track3
	call play_track

    ld   a,ixl                  ; save the register 7 of the track 3.
    ex   af,af'

    ld   iy,ply_psgregistersarray + 2
	ld ix,track2
	call play_track
	
    ex   af,af'
    add  a,a                    ; mix reg7 from track2 with track3, making room first.
    or   ixl
    rla
    ex   af,af'
	
    ld iy,ply_psgregistersarray
	ld ix,track1
	call play_track

    ex   af,af'
    or   ixl                    ; mix reg7 from track3 with track2+1.

; ---------------------------------------------------------------------------

; there are two holes in the list, because the volume registers are set relatively to the frequency of the same channel (+7, always).
; also, the reg7 is passed as a register, so is not kept in the memory.
ply_psgregistersarray
ply_psgreg0 defb 0
ply_psgreg1 defb 0
ply_psgreg2 defb 0
ply_psgreg3 defb 0
ply_psgreg4 defb 0
ply_psgreg5 defb 0
ply_psgreg6 defb 0
ply_psgreg8 defb 0              ; +7
        defb 0
ply_psgreg9 defb 0              ; +9
        defb 0
ply_psgreg10 defb 0             ; +11
ply_psgreg11 defb 0
ply_psgreg12 defb 0
ply_psgreg13 defb 0
ply_psgregistersarray_end



; send the registers to psg. various codes according to the machine used.
ply_sendregisters
; a=register 7

; ---------------------------------------------------------------------------
ext_psg_select      equ $f984   ; select external ay/ym register
ext_psg_write       equ $f884   ; write value in selected register in external ay/ym 

; send registers to external ay
use_external_ay
    ld   hl,ply_psgregistersarray
    ld   e,a                    ; save value for psg r7
    ld   bc,ext_psg_select
; register 0
    xor  a
    out  (c),a                  ; register
    outi                        ; value
    inc  b
    
; register 1
    inc  a
    out  (c),a
    outi
    inc  b

; register 2
    inc  a
    out  (c),a
    outi
    inc  b

; register 3
    inc  a
    out  (c),a
    outi
    inc  b

; register 4
    inc  a
    out  (c),a
    outi
    inc  b

; register 5
    inc  a
    out  (c),a
    outi
    inc  b

; register 6
    inc  a
    out  (c),a
    outi
    inc  b

; register 7
    inc  a
    out  (c),a
    dec  b
    out  (c),e                  ; use the stored register 7
    inc  b

; register 8
    if   ply_usefades
        ; with fade
        ld   d,8
        out  (c),d
        ld   a,(hl)
        inc  hl                 ; skip volume byte
ply_channel1_fadevalue_ext
        sub  0                  ; set a value from 0 (full volume) to 16 or more (volume to 0).
        jr   nc,.apply_fade_channel
        xor  a
.apply_fade_channel
        dec  b
        out  (c),a
        inc  b
        inc  hl                 ; skip unused byte.
    else
        ; no fade
        inc  a
        out  (c),a
        outi
        inc  hl                 ; skip unused byte (*)
        inc  b
    endif

; register 9
    if   ply_usefades
        ; with fade
        inc  d
        out  (c),d
        ld   a,(hl)
        inc  hl                 ; skip volume byte
ply_channel2_fadevalue_ext
        sub  0                  ; set a value from 0 (full volume) to 16 or more (volume to 0).
        jr   nc,.apply_fade_channel
        xor  a
.apply_fade_channel
        dec  b
        out  (c),a
        inc  b
        inc  hl                 ; skip unused byte.
    else
        ; no fade
        inc  a
        out  (c),a
        outi
        inc  hl                 ; skip unused byte
        inc  b
    endif
    
; register 10
    if   ply_usefades
        ; with fade
        inc  d
        out  (c),d
        ld   a,(hl)
        inc  hl                 ; skip volume byte
ply_channel3_fadevalue_ext
        sub  0                  ; set a value from 0 (full volume) to 16 or more (volume to 0).
        jr   nc,.apply_fade_channel
        xor  a
.apply_fade_channel
        dec  b
        out  (c),a
        inc  b
        inc  hl                 ; skip unused byte.
        ld   a,d                ; update ay register counter
    else
        ; no fade
        inc  a
        out  (c),a
        outi
        inc  hl                 ; skip unused byte
        inc  b
    endif

; register 11
    inc  a
    out  (c),a
    outi
    inc  b

; register 12
    inc  a
    out  (c),a
    outi
    inc  b

; register 13
ply_psgreg13_code_ext
    inc  a
    out  (c),a
    ld   a,(hl)
.ply_psgreg13_retrig
    cp   255                    ; if isretrig?, force the r13 to be triggered.
    ret  z
    outi                        ; = dec b | out (c),a
    ld   (.ply_psgreg13_retrig + 1),a
    ret

play_internal_psg:
    ld   de,$c080
    ld   b,$f6
    out  (c),d                  ; $f6c0
    exx
    ld   hl,ply_psgregistersarray
    ld   e,$f6
    ld   bc,$f401

; register 0
    defb $ed,$71                ; $f400+register
    ld   b,e
    defb $ed,$71                ; $f600
    dec  b
    outi                        ; $f400+value
    exx
    out  (c),e                  ; $f680
    out  (c),d                  ; $f6c0
    exx

; register 1
    out  (c),c
    ld   b,e
    defb $ed,$71
    dec  b
    outi
    exx
    out  (c),e
    out  (c),d
    exx
    inc  c

; register 2
    out  (c),c
    ld   b,e
    defb $ed,$71
    dec  b
    outi
    exx
    out  (c),e
    out  (c),d
    exx
    inc  c

; register 3
    out  (c),c
    ld   b,e
    defb $ed,$71
    dec  b
    outi
    exx
    out  (c),e
    out  (c),d
    exx
    inc  c

; register 4
    out  (c),c
    ld   b,e
    defb $ed,$71
    dec  b
    outi
    exx
    out  (c),e
    out  (c),d
    exx
    inc  c

; register 5
    out  (c),c
    ld   b,e
    defb $ed,$71
    dec  b
    outi
    exx
    out  (c),e
    out  (c),d
    exx
    inc  c

; register 6
    out  (c),c
    ld   b,e
    defb $ed,$71
    dec  b
    outi
    exx
    out  (c),e
    out  (c),d
    exx
    inc  c

; register 7
    out  (c),c
    ld   b,e
    defb $ed,$71
    dec  b
    dec  b
    out  (c),a                  ; read a register instead of the list.
    exx
    out  (c),e
    out  (c),d
    exx
    inc  c

; register 8
    out  (c),c
    ld   b,e
    defb $ed,$71
    dec  b
    if ply_usefades
    dec  b
    ld   a,(hl)
ply_channel1_fadevalue_int
    sub  0                      ; set a value from 0 (full volume) to 16 or more (volume to 0).
    jr   nc,$+6
    defb $ed,$71
    jr   $+4
    out  (c),a
    inc  hl

    else
    
    outi
    endif
    exx
    out  (c),e
    out  (c),d
    exx
    inc  c
    inc  hl                     ; skip unused byte.

; register 9
    out  (c),c
    ld   b,e
    defb $ed,$71
    dec  b
    if ply_usefades             ; if ply_usefades is set to 1, we manage the volume fade.
    dec  b
    ld   a,(hl)
ply_channel2_fadevalue_int 
    sub  0                      ; set a value from 0 (full volume) to 16 or more (volume to 0).
    jr   nc,$+6
    defb $ed,$71
    jr   $+4
    out  (c),a
    inc  hl

    else
    
    outi
    endif
    exx
    out  (c),e
    out  (c),d
    exx
    inc  c
    inc  hl                     ; skip unused byte.

; register 10
    out  (c),c
    ld   b,e
    defb $ed,$71
    dec  b
    if ply_usefades
    dec  b
    ld   a,(hl)
ply_channel3_fadevalue_int
    sub  0                      ; set a value from 0 (full volume) to 16 or more (volume to 0).
    jr   nc,$+6
    defb $ed,$71
    jr   $+4
    out  (c),a
    inc  hl

    else
    
    outi
    endif
    exx
    out  (c),e
    out  (c),d
    exx
    inc  c

; register 11
    out  (c),c
    ld   b,e
    defb $ed,$71
    dec  b
    outi
    exx
    out  (c),e
    out  (c),d
    exx
    inc  c

; register 12
    out  (c),c
    ld   b,e
    defb $ed,$71
    dec  b
    outi
    exx
    out  (c),e
    out  (c),d
    exx
    inc  c

; register 13
ply_psgreg13_code_int
    ld   a,(hl)
.ply_psgreg13_retrig
    cp   255                    ; if isretrig?, force the r13 to be triggered.
    ret  z
    ld   (.ply_psgreg13_retrig + 1),a
    out  (c),c
    ld   b,e
    defb $ed,$71
    dec  b
    outi
    exx
    out  (c),e
    out  (c),d
    ret


; ---------------------------------------------------------------------------
; plays a sound stream.
; hl=pointer on instrument data
; iy=pointer on register code (volume, frequency).
; e=note
; d=inverted volume
; de'=trackpitch

; ret=
; hl=new instrument pointer.
; ixl=reg7 mask (x00x)

; also used inside =
; b,c=read byte/second byte.
; ixh=save original note (only used for independant mode).
; ---------------------------------------------------------------------------
ply_playsound
    ld   b,(hl)
    inc  hl
    rr   b
    jp   c,ply_ps_hard

;---------------
; software sound
;---------------
    ; second byte needed?
    rr   b
    jr   c,ply_ps_s_secondbyteneeded

    ; no second byte needed. we need to check if volume is null or not.
    ld   a,b
    and  %1111
    jr   nz,ply_ps_s_soundon

    ; null volume. it means no sound. we stop the sound, the noise, and it's over.
    ld   (iy + 7),a             ; we have to make the volume to 0, because if a bass hard was activated before, we have to stop it.
    ld   ixl,%1001

    ret

; ---------------------------------------------------------------------------
ply_ps_s_soundon
    ; volume is here, no second byte needed. it means we have a simple software sound (sound = on, noise = off)
    ; we have to test arpeggio and pitch, however.
    ld   ixl,%1000

    sub  d                      ; code volume.
    jr   nc,.skip_xor_a
    xor  a
.skip_xor_a
    ld   (iy + 7),a

    rr   b                      ; needed for the subroutine to get the good flags.
    call ply_ps_calculatefrequency
    ld   (iy + 0),l             ; code frequency.
    ld   (iy + 1),h
    exx

    ret

; ---------------------------------------------------------------------------
ply_ps_s_secondbyteneeded
    ld   ixl,%1000              ; by defaut, no noise, sound.

    ; second byte needed.
    ld   c,(hl)
    inc  hl

    ; noise?
    ld   a,c
    and  %11111
    jr   z,ply_ps_s_sbn_nonoise
    ld   (ply_psgreg6),a
    ld   ixl,%0000              ; open noise channel.
ply_ps_s_sbn_nonoise

    ; here we have either volume and/or sound. so first we need to read the volume.
    ld   a,b
    and  %1111
    sub  d                      ; code volume.
    jr   nc,.skip_xor_a2
    xor  a
.skip_xor_a2
    ld   (iy + 7),a

    ; sound?
    bit  5,c
    jr   nz,ply_ps_s_sbn_sound
    ; no sound. stop here.
    inc  ixl                    ; set sound bit to stop the sound.
    ret

; ---------------------------------------------------------------------------
ply_ps_s_sbn_sound
    ; manual frequency?
    rr   b                      ; needed for the subroutine to get the good flags.
    bit  6,c
    call ply_ps_calculatefrequency_testmanualfrequency
    ld   (iy + 0),l             ; code frequency.
    ld   (iy + 1),h
    exx

    ret

; ---------------------------------------------------------------------------

;-----------
; hard sound
;-----------
ply_ps_hard
    ; we don't set the volume to 16 now because we may have reached the end of the sound!

    rr   b                      ; test retrig here, it is common to every hard sounds.
    jr   nc,ply_ps_hard_noretrig
    ld   a,(ply_track1_instrumentspeedcpt + 1)  ; retrig only if it is the first step in this line of instrument!
    ld   c,a
    ld   a,(ply_track1_instrumentspeed + 1)
    cp   c
    jr   nz,ply_ps_hard_noretrig
    ld   a,ply_retrigvalue
.sm_retrig
    ld   (ply_psgreg13_code_ext.ply_psgreg13_retrig + 1),a
ply_ps_hard_noretrig

    ; independant/loop or software/hardware dependent?
    bit  1,b                    ; we don't shift the bits, so that we can use the same code (frequency calculation) several times.
    jp   nz,ply_ps_hard_looporindependent

    ; hardware sound.
    ld   (iy + 7),16            ; set volume
    ld   ixl,%1000              ; sound is always on here (only independence mode can switch it off).

    ; this code is common to both software and hardware dependent.
    ld   c,(hl)                 ; get second byte.
    inc  hl
    ld   a,c                    ; get the hardware envelope waveform.
    and  %1111                  ; we don't care about the bit 7-4, but we have to clear them, else the waveform might be reset.
    ld   (ply_psgreg13),a

    bit  0,b
    jr   z,ply_ps_hardwaredependent

;-------------------
; software dependent
;-------------------

    ; calculate the software frequency
    bit  4-2,b                  ; manual frequency? -2 because the byte has been shifted previously.
    call ply_ps_calculatefrequency_testmanualfrequency
    ld   (iy + 0),l             ; code software frequency.
    ld   (iy + 1),h
    exx

    ; shift the frequency.
    ld   a,c
    rra
    rra                         ; shift=shift*4. the shift is inverted in memory (7 - editor shift).
    and  %11100
    ld   (ply_ps_sd_shift + 1),a
    ld   a,b                    ; used to get the hardwarepitch flag within the second registers set.
    exx

ply_ps_sd_shift
    jr   .smjr_ply_ps_sd_shift
.smjr_ply_ps_sd_shift
    srl  h
    rr   l
    srl  h
    rr   l
    srl  h
    rr   l
    srl  h
    rr   l
    srl  h
    rr   l
    srl  h
    rr   l
    srl  h
    rr   l
    jr   nc,.skip_inc_hl
    inc  hl
.skip_inc_hl

    ; hardware pitch?
    bit  7-2,a
    jr   z,ply_ps_sd_nohardwarepitch
    exx                         ; get pitch and add it to the just calculated hardware frequency.
    ld   a,(hl)
    inc  hl
    exx
    add  a,l                    ; slow. can be optimised? probably never used anyway.....
    ld   l,a
    exx
    ld   a,(hl)
    inc  hl
    exx
    adc  a,h
    ld   h,a
ply_ps_sd_nohardwarepitch
    ld   (ply_psgreg11),hl
    exx

    ; this code is also used by hardware dependent.
ply_ps_sd_noise
    ; noise?
    bit  7,c
    ret  z
    ld   a,(hl)
    inc  hl
    ld   (ply_psgreg6),a
    ld   ixl,%0000
    ret

; ---------------------------------------------------------------------------

;-------------------
; hardware dependent
;-------------------
ply_ps_hardwaredependent
    ; calculate the hardware frequency
    bit  4-2,b                  ; manual hardware frequency? -2 because the byte has been shifted previously.
    call ply_ps_calculatefrequency_testmanualfrequency
    ld   (ply_psgreg11),hl      ; code hardware frequency.
    exx

    ; shift the hardware frequency.
    ld   a,c
    rra
    rra                         ; shift=shift*4. the shift is inverted in memory (7 - editor shift).
    and  %11100
    ld   (ply_ps_hd_shift + 1),a
    ld   a,b                    ; used to get the software flag within the second registers set.
    exx

ply_ps_hd_shift
    jr   .smjr_ply_ps_hd_shift
.smjr_ply_ps_hd_shift
    sla  l
    rl   h
    sla  l
    rl   h
    sla  l
    rl   h
    sla  l
    rl   h
    sla  l
    rl   h
    sla  l
    rl   h
    sla  l
    rl   h

    ; software pitch?
    bit  7-2,a
    jr   z,ply_ps_hd_nosoftwarepitch
    exx                         ; get pitch and add it to the just calculated software frequency.
    ld   a,(hl)
    inc  hl
    exx
    add  a,l
    ld   l,a                    ; slow. can be optimised? probably never used anyway.....
    exx
    ld   a,(hl)
    inc  hl
    exx
    adc  a,h
    ld   h,a
ply_ps_hd_nosoftwarepitch
    ld   (iy + 0),l             ; code frequency.
    ld   (iy + 1),h
    exx

    ; go to manage noise, common to software dependent.
    jr   ply_ps_sd_noise

; ---------------------------------------------------------------------------
ply_ps_hard_looporindependent
    bit  0,b                    ; we mustn't shift it to get the result in the carry, as it would be mess the structure
    jr   z,ply_ps_independent   ; of the flags, making it uncompatible with the common code.

    ; the sound has ended.
    ; if sound effects activated, we mark the "end of sound" by returning a 0 as an address.
    if ply_usesoundeffects
ply_ps_endsound_sfx
    ld   a,0                    ; is the sound played is a sfx (1) or a normal sound (0)?
    or   a
    jr   z,ply_ps_endsound_notasfx

;    ld   hl,0
;    ret

	;******* for fano
	; we have an ending sound effect.
	; the only way to know if it should stop or loop is to compare its 'jump' position
	; with the "empty" instrument.
	; we return 0 if the sound should stop.
	ld   a,(hl)
	inc  hl
	ld   h,(hl)
	ld   l,a

	ld   (ply_ps_save_hl + 1),hl
	push de

empty_instrument_data_address
    ld   de,0
	or   a
	sbc  hl,de
	pop  de
	ret  z	                    ; the sound should stop!

	; the sound should loop. we return its loop point address.
ply_ps_save_hl 
    ld   hl,0
	jp   ply_playsound
	;****************

ply_ps_endsound_notasfx
    endif

    ; the sound has ended. read the new pointer and restart instrument.

    ld   a,(hl)
    inc  hl
    ld   h,(hl)
    ld   l,a
    jp   ply_playsound

; ---------------------------------------------------------------------------

;------------
; independent
;------------
ply_ps_independent
    ld   (iy + 7),16            ; set volume

    ; sound?
    bit  7-2,b                  ; -2 because the byte has been shifted previously.
    jr   nz,ply_ps_i_soundon
    ; no sound! it means we don't care about the software frequency (manual frequency, arpeggio, pitch).
    ld   ixl,%1001
    jr   ply_ps_i_skipsoftwarefrequencycalculation
ply_ps_i_soundon
    ld   ixl,%1000              ; sound is on.
    ld   ixh,e                  ; save the original note for the hardware frequency, because a software arpeggio will modify it.

    ; calculate the software frequency
    bit  4-2,b                  ; manual frequency? -2 because the byte has been shifted previously.
    call ply_ps_calculatefrequency_testmanualfrequency
    ld   (iy + 0),l             ; code software frequency.
    ld   (iy + 1),h
    exx

    ld   e,ixh
ply_ps_i_skipsoftwarefrequencycalculation

    ld   b,(hl)                 ; get second byte.
    inc  hl
    ld   a,b                    ; get the hardware envelope waveform.
    and  %1111                  ; we don't care about the bit 7-4, but we have to clear them, else the waveform might be reset.
    ld   (ply_psgreg13),a

    ; calculate the hardware frequency
    rr   b                      ; must shift it to match the expected data of the subroutine.
    rr   b
    bit  4-2,b                  ; manual hardware frequency? -2 because the byte has been shifted previously.
    call ply_ps_calculatefrequency_testmanualfrequency
    ld   (ply_psgreg11),hl      ; code hardware frequency.
    exx

    ; noise? we can't use the previous common code, because the setting of the noise is different, since independent can have no sound.
    bit  7-2,b
    ret  z
    ld   a,(hl)
    inc  hl
    ld   (ply_psgreg6),a
    ld   a,ixl                  ; set the noise bit.
    res  3,a
    ld   ixl,a
    ret

; ---------------------------------------------------------------------------
; subroutine that =
; if manual frequency? (flag z off), read frequency (word) and adds the trackpitch (de').
; else, auto frequency.
;     if arpeggio? = 1 (bit 3 from b), read it (byte).
;     if pitch? = 1 (bit 4 from b), read it (word).
;     calculate the frequency according to the note (e) + arpeggio + trackpitch (de').

; hl = pointer on instrument data.
; de'= trackpitch.

; ret=
; hl = pointer on instrument moved forward.
; hl'= frequency
;     return in auxiliary registers
; ---------------------------------------------------------------------------
ply_ps_calculatefrequency_testmanualfrequency
    jr   z,ply_ps_calculatefrequency
    ; manual frequency. we read it, no need to read pitch and arpeggio.
    ; however, we add trackpitch to the read frequency, and that's all.
    ld   a,(hl)
    inc  hl
    exx
    add  a,e                    ; add trackpitch lsb.
    ld   l,a
    exx
    ld   a,(hl)
    inc  hl
    exx
    adc  a,d                    ; add trackpitch hsb.
    ld   h,a
    ret

; ---------------------------------------------------------------------------
ply_ps_calculatefrequency
    ; pitch?
    bit  5-1,b
    jr   z,ply_ps_s_soundon_nopitch
    ld   a,(hl)
    inc  hl
    exx
    add  a,e                    ; if pitch found, add it directly to the trackpitch.
    ld   e,a
    exx
    ld   a,(hl)
    inc  hl
    exx
    adc  a,d
    ld   d,a
    exx
ply_ps_s_soundon_nopitch

    ; arpeggio?
    ld   a,e
    bit  4-1,b
    jr   z,ply_ps_s_soundon_arpeggioend
    add  a,(hl)                 ; add arpeggio to note.
    inc  hl
    cp   144
    jr   c,ply_ps_s_soundon_arpeggioend
    ld   a,143
ply_ps_s_soundon_arpeggioend

    ; frequency calculation.
    exx

    ld   l,a
    ld   h,0
    add  hl,hl
    
    ld   bc,ply_frequencytable
    add  hl,bc

    ld   a,(hl)
    inc  hl
    ld   h,(hl)
    ld   l,a
    add  hl,de                  ; add trackpitch + instrumentpitch (if any).

    ret

; ---------------------------------------------------------------------------
; read one track.
; hl=track pointer.

; ret =
; hl=new track pointer.
; carry = 1 = wait a lines. carry=0=line not empty.
; a=wait (0(=256)-127), if carry.
; d=parameters + volume.
; e=note
; b=instrument. 0=rst
; ix=pitchadd. only used if pitch? = 1.
; ---------------------------------------------------------------------------
ply_readtrack
    ld   a,(hl)
    inc  hl
    srl  a                      ; full optimisation? if yes = note only, no pitch, no volume, same instrument.
    jr   c,ply_readtrack_fulloptimisation
    sub  32                     ; 0-31 = wait.
    jr   c,ply_readtrack_wait
    jr   z,ply_readtrack_nooptimisation_escapecode
    dec  a                      ; 0 (32-32) = escape code for more notes (parameters will be read)
    
    ; note. parameters are present. but the note is only present if note? flag is 1.
    ld   e,a                    ; save note.

    ; read parameters
ply_readtrack_readparameters
    ld   a,(hl)
    ld   d,a                    ; save parameters.
    inc  hl

    rla                         ; pitch?
    jr   nc,ply_readtrack_pitch_end
    ld   b,(hl)                 ; get pitchadd
    ld   ixl,b
    inc  hl
    ld   b,(hl)
    ld   ixh,b
    inc  hl
ply_readtrack_pitch_end

    rla                         ; skip isnote? flag.
    rla                         ; new instrument?
    ret  nc
    ld   b,(hl)
    inc  hl
    or   a                      ; remove carry, as the player interpret it as a wait command.
    ret

; ---------------------------------------------------------------------------
; escape code, read the note and returns to read the parameters.
; ---------------------------------------------------------------------------
ply_readtrack_nooptimisation_escapecode
    ld   e,(hl)
    inc  hl
    jr   ply_readtrack_readparameters

; ---------------------------------------------------------------------------
ply_readtrack_fulloptimisation
    ; note only, no pitch, no volume, same instrument.
    ld   d,%01000000            ; note only.
    sub  1
    ld   e,a
    ret  nc
    ld   e,(hl)                 ; escape code found (0). read note.
    inc  hl
    or   a
    ret

; ---------------------------------------------------------------------------
ply_readtrack_wait
    add  a,32
    ret

; ---------------------------------------------------------------------------
ply_frequencytable
    dw 3822,3608,3405,3214,3034,2863,2703,2551,2408,2273,2145,2025 ; oct -3
    dw 1911,1804,1703,1607,1517,1432,1351,1276,1204,1136,1073,1012 ; oct -2 (0 in cpc32)
    dw 956,902,851,804,758,716,676,638,602,568,536,506             ; oct -1 (0 in cpc16, st32)
    dw 478,451,426,402,379,358,338,319,301,284,268,253             ; oct  0 (0 in st16, arc32)
    dw 239,225,213,201,190,179,169,159,150,142,134,127             ; oct  1 (0 in arc16)
    dw 119,113,106,100,95,89,84,80,75,71,67,63                     ; oct  2
    dw 60,56,53,50,47,45,42,40,38,36,34,32                         ; oct  3
    dw 30,28,27,25,24,22,21,20,19,18,17,16                         ; oct  4
    dw 15,14,13,13,12,11,11,10,9,9,8,8                             ; oct  5 (medium useful)
    dw 7,7,7,6,6,6,5,5,5,4,4,4                                     ; oct  6 (worst)
    dw 4,4,3,3,3,3,3,2,2,2,2,2                                     ; oct  7 (even worst)
    dw 2,2,2,2,1,1,1,1,1,1,1,1                                     ; oct  8 (not useful)

; ---------------------------------------------------------------------------
; rutina de inicialización de la canción
; entradas:
;    de = music
; ---------------------------------------------------------------------------
ply_init
    ld   hl,9                   ; skip header, samplechannel, ym clock (db*3), and replay frequency.
    add  hl,de

	ld a,(hl)
	inc hl
	ld (ix+ply_speed),a
    ld   c,(hl)                 ; get instruments chunk size.
    inc  hl
    ld   b,(hl)
    inc  hl
	ld (ix+ply_track_instrumentstablept+track1_offs+0),l
	ld (ix+ply_track_instrumentstablept+track1_offs+1),h
	ld (ix+ply_track_instrumentstablept+track2_offs+0),l
	ld (ix+ply_track_instrumentstablept+track2_offs+1),h
	ld (ix+ply_track_instrumentstablept+track3_offs+0),l
	ld (ix+ply_track_instrumentstablept+track3_offs+1),h

    add  hl,bc                  ; skip instruments to go to the linker address.
    ; get the pre-linker information of the first pattern.

	ld a,(hl)
	inc hl
	ld (ix+ply_height),a

	ld a,(hl)
	inc hl
	ld (ix+ply_track_transposition+track1_offs),a
	ld a,(hl)
	inc hl
	ld (ix+ply_track_transposition+track2_offs),a
	ld a,(hl)
	inc hl
	ld (ix+ply_track_transposition+track3_offs),a

	ld a,(hl)
	inc hl
	ld (ix+ply_specialtrack_pt+0),l
	ld a,(hl)
	inc hl
	ld (ix+ply_specialtrack_pt+1),h
	
    ld   (ix+ply_linker_pt + 0),l ; get the linker address.
    ld   (ix+ply_linker_pt + 1),h ; get the linker address.

    ld   a,1
    ld   (ix+ply_speedcpt),a
    ld   (ix+ply_heightcpt),a

    ld   a,$ff
    ld   (ply_psgreg13),a
    
    ; set the instruments pointers to instrument 0 data (header has to be skipped).
    ld   hl,(ply_track1_instrumentstablept + 1)
    ld   e,(hl)
    inc  hl
    ld   d,(hl)
    ex   de,hl
    inc  hl                     ; skip instrument 0 header.
    inc  hl
    ld   (ply_track1_instrument + 1),hl
    ld   (ply_track2_instrument + 1),hl
    ld   (ply_track3_instrument + 1),hl
    ret

; ---------------------------------------------------------------------------
; stop the music, cut the channels.
; ---------------------------------------------------------------------------
ply_stop
    ld   hl,ply_psgreg8
    ld   bc,$0300               ; bug: no silenciaba los 3 canales
.loop_ply_stop
    ld   (hl),c
    inc  hl
    inc  hl                     ; extra inc hl to fix it
    djnz ply_stop
    ld   a,%00111111
    jp   ply_sendregisters

    if ply_usesoundeffects
; ---------------------------------------------------------------------------
; initialize the sound effects.
; de = sfx music.
; ---------------------------------------------------------------------------
ply_sfx_init
    ; find the instrument table.
    ld   hl,12
    add  hl,de
    ld   (ply_sfx_play_instrumenttable + 1),hl

	;******* for fano
	ld   e,(hl)
	inc  hl
	ld   d,(hl)
	ex   de,hl

	inc  hl					    ; skip instrument 0 header.
	inc  hl

	ld   (empty_instrument_data_address + 1),hl
	;****************
    
; clear the three channels of any sound effect.
ply_sfx_stopall
    ld   hl,0
    ld   (ply_sfx_track1_instrument + 1),hl
    ld   (ply_sfx_track2_instrument + 1),hl
    ld   (ply_sfx_track3_instrument + 1),hl
    ret

ply_sfx_offsetpitch         equ 0
ply_sfx_offsetvolume        equ ply_sfx_track1_volume - ply_sfx_track1_pitch
ply_sfx_offsetnote          equ ply_sfx_track1_note - ply_sfx_track1_pitch
ply_sfx_offsetinstrument    equ ply_sfx_track1_instrument - ply_sfx_track1_pitch
ply_sfx_offsetspeed         equ ply_sfx_track1_instrumentspeed - ply_sfx_track1_pitch
ply_sfx_offsetspeedcpt      equ ply_sfx_track1_instrumentspeedcpt - ply_sfx_track1_pitch

; ---------------------------------------------------------------------------
; plays a sound effects along with the music.
; entradas:
;    a = no channel (0,1,2)
;    l = sfx number (>0)
;    h = volume (0...f)
;    e = note (0...143)
;    d = speed (0 = as original, 1...255 = new speed (1 is fastest))
;   bc = inverted pitch (-#ffff -> ffff). 0 is no pitch. the higher the pitch, the lower the sound.
; ---------------------------------------------------------------------------
ply_sfx_play
    ld   ix,ply_sfx_track1_pitch
    or   a
    jr   z,ply_sfx_play_selected
    ld   ix,ply_sfx_track2_pitch
    dec  a
    jr   z,ply_sfx_play_selected
    ld   ix,ply_sfx_track3_pitch
    
ply_sfx_play_selected
    ld   (ix + ply_sfx_offsetpitch + 1),c   ; set pitch
    ld   (ix + ply_sfx_offsetpitch + 2),b
    ld   a,e                    ; set note
    ld   (ix + ply_sfx_offsetnote),a
    ld   a,15                   ; set volume
    sub  h
    ld   (ix + ply_sfx_offsetvolume),a
    ld   h,0                    ; set instrument address
    add  hl,hl
ply_sfx_play_instrumenttable
    ld   bc,0
    add  hl,bc
    ld   a,(hl)
    inc  hl
    ld   h,(hl)
    ld   l,a
    ld   a,d                    ; read speed or use the user's one?
    or   a
    jr   nz,ply_sfx_play_userspeed
    ld   a,(hl)                 ; get speed
ply_sfx_play_userspeed
    ld   (ix + ply_sfx_offsetspeed + 1),a
    ld   (ix + ply_sfx_offsetspeedcpt + 1),a
    inc  hl                     ; skip retrig
    inc  hl
    ld   (ix + ply_sfx_offsetinstrument + 1),l
    ld   (ix + ply_sfx_offsetinstrument + 2),h

    ret

; ---------------------------------------------------------------------------
; stops a sound effect on the selected channel                    *** syx ***
; a = no channel (0,1,2)
; ---------------------------------------------------------------------------
ply_sfx_stop
    ld   hl,ply_sfx_track1_instrument + 1
    or   a
    jr   z,ply_sfx_stop_channelfound
    ld   hl,ply_sfx_track2_instrument + 1
    dec  a
    jr   z,ply_sfx_stop_channelfound
    ld   hl,ply_sfx_track3_instrument + 1
    dec  a

ply_sfx_stop_channelfound
    ld   (hl),a
    inc  hl
    ld   (hl),a
    ret

    endif

    if ply_usefades
; ---------------------------------------------------------------------------
; sets the fade value.
; a = fade value (0 = full volume, 16 or more = no volume).
; ---------------------------------------------------------------------------
ply_setfadevalue
.sm_fadevalue_01
    ld   (ply_channel1_fadevalue_ext + 1),a
.sm_fadevalue_02
    ld   (ply_channel2_fadevalue_ext + 1),a
.sm_fadevalue_03
    ld   (ply_channel3_fadevalue_ext + 1),a
	ret

    endif

; ---------------------------------------------------------------------------
; choose ay used for the music, the other is used for sfx
; a = 0 -> external ay | rest -> internal ay
; ---------------------------------------------------------------------------
ply_selectmusicay
    or   a
    jr   nz,.set_internal_ay
.set_external_ay
    ; patch send registers to ay
    ld   a,$21                  ; ld hl,$xxxx
    ld   hl,ply_psgregistersarray
    ld   (ply_sendregisters),a
    ld   (ply_sendregisters + 1),hl
    
    ; patch psg r13 retrig
    ld   hl,ply_psgreg13_code_ext.ply_psgreg13_retrig + 1
    ld   (ply_track1_instrumentstablept.sm_retrig + 1),hl
    ld   (ply_track2_instrumentstablept.sm_retrig + 1),hl
    ld   (ply_track3_instrumentstablept.sm_retrig + 1),hl
    ld   (ply_ps_hard.sm_retrig + 1), hl
    
    ; patch fade
    if ply_usefades
    ld   hl,ply_channel1_fadevalue_ext + 1
    ld   (ply_setfadevalue.sm_fadevalue_01 + 1),hl
    ld   hl,ply_channel2_fadevalue_ext + 1
    ld   (ply_setfadevalue.sm_fadevalue_02 + 1),hl
    ld   hl,ply_channel3_fadevalue_ext + 1
    ld   (ply_setfadevalue.sm_fadevalue_03 + 1),hl
    endif
    ret

.set_internal_ay
    ; patch send registers to ay
    ld   a,$c3                  ; jp $xxxx
    ld   hl,use_internal_ay
    ld   (ply_sendregisters),a
    ld   (ply_sendregisters + 1),hl
    
    ; patch psg r13 retrig
    ld   hl,ply_psgreg13_code_int.ply_psgreg13_retrig + 1
    ld   (ply_track1_instrumentstablept.sm_retrig + 1),hl
    ld   (ply_track2_instrumentstablept.sm_retrig + 1),hl
    ld   (ply_track3_instrumentstablept.sm_retrig + 1),hl
    ld   (ply_ps_hard.sm_retrig + 1), hl

    if ply_usefades
    ld   hl,ply_channel1_fadevalue_int + 1
    ld   (ply_setfadevalue.sm_fadevalue_01 + 1),hl
    ld   hl,ply_channel2_fadevalue_int + 1
    ld   (ply_setfadevalue.sm_fadevalue_02 + 1),hl
    ld   hl,ply_channel3_fadevalue_int + 1
    ld   (ply_setfadevalue.sm_fadevalue_03 + 1),hl
    endif

    ret

;------------------
; read the track 1.
;------------------
read_track:
; store the parameters, because the player below is called every frame, but the read track isn't.
	ld a,(ix+ply_track_waitcounter)
    dec  a
    jr   nz,ply_track1_newinstrument_setwait

	ld l,(ix+ply_track_pt+0)
	ld h,(ix+ply_track_pt+1)
    call ply_readtrack
    ld   (ix+ply_track_pt+0),l
	ld 	(ix+ply_track_pt+1),h
    jr   c,ply_track1_newinstrument_setwait

    ; no wait command. can be a note and/or effects.
    ld   a,d                    ; make a copy of the flags+volume in a, not to temper with the original.

    rra                         ; volume? if bit 4 was 1, then volume exists on b3-b0
    jr   nc,ply_track1_samevolume
    and  %1111
    ld   (ix+ply_track_volume),a
ply_track1_samevolume

    rl   d                      ; new pitch?
    jr   nc,ply_track1_nonewpitch
    ld   (ply_track1_pitchadd + 1),ix
ply_track1_nonewpitch

    rl   d                      ; note? if no note, we don't have to test if a new instrument is here.
    jr   nc,ply_track1_nonotegiven
    ld   a,e
	add a,(ix+ply_track_transposition)
    ld   (ix+ply_track_note),a

    ld   hl,0                   ; reset the trackpitch.
    ld   (ix+ply_track_pitch + 0),l
	ld 	(ix+ply_track_pitch+1),h

    rl   d                      ; new instrument?
    jr   c,ply_track1_newinstrument

	ply_track1_saveptinstrument 
    ld   hl,0                   ; same instrument. we recover its address to restart it.
  
	ld   a,(ix+ply_track1_instrumentspeed)     ; reset the instrument speed counter. never seemed useful...
    ld   (ix+ply_track1_instrumentspeedcpt),a
    
	jr   ply_track1_instrumentresetpt

ply_track1_newinstrument        ; new instrument. we have to get its new address, and speed.
    ld   l,b                    ; h is already set to 0 before.
    add  hl,hl
	ld c,(ix+ply_track_instrumentstablept+0)
	ld b,(ix+ply_track_instrumentstablept+1)
    add  hl,bc
    ld   a,(hl)                 ; get instrument address.
    inc  hl
    ld   h,(hl)
    ld   l,a
    ld   a,(hl)                 ; get instrument speed.
    inc  hl
    ld   (ix + ply_track1_instrumentspeed),a
    ld   (ix + ply_track1_instrumentspeedcpt),a
    ld   a,(hl)
    or   a                      ; get isretrig?. code it only if different to 0, else next instruments are going to overwrite it.
    jr   z,.skip_retrig
.sm_retrig
    ld   (ply_psgreg13_code_ext.ply_psgreg13_retrig + 1),a
.skip_retrig

    inc  hl

    ld   (ix + ply_track1_saveptinstrument + 0),l   ; when using the instrument again, no need to give the speed, it is skipped.
    ld   (ix + ply_track1_saveptinstrument + 1),h   ; when using the instrument again, no need to give the speed, it is skipped.

ply_track1_instrumentresetpt
    ld   (ix + ply_track_instrument + 0),l
    ld   (ix + ply_track_instrument + 1),h

ply_track1_nonotegiven

    ld   a,1
ply_track1_newinstrument_setwait
    ld   (ix+ply_track_waitcounter),a
	ret
	
		
play_track:
	ld l,(ix+ply_track_pitch+0)
	ld h,(ix+ply_track_pitch+1)
	ld e,(ix+ply_track_pitch_add+0)
	ld d,(ix+ply_track_pitch_add+1)
    add  hl,de
	ld (ix+ply_track_pitch+0),l
	ld (ix+ply_track_pitch+1),h
    sra  h                      ; shift the pitch to slow its speed.
    rr   l
    sra  h
    rr   l
    ex   de,hl
    exx
	ld d,(ix+ply_track_volume)
	ld e,(ix+ply_track_note)
	
	ld l,(ix+ply_track_instrument+0)
	ld h,(ix+ply_track_instrument+1)
    call ply_playsound

	ld a,(ix+ply_track_instrument_speed_cpt)
    dec  a
    jr   nz,ply_track3_playnoforward
	
	ld (ix+ply_track_instrument+0),l
	ld (ix+ply_track_instrument+1),h
	
	ld a,(ix+ply_track_instrumentspeed)
ply_track3_playnoforward
    ld   (ix+ply_track_instrumentspeedcpt),a

;-----------------------------------------------------------------------------------------
; play sound effects on track 3 (only assembled used if ply_usesoundeffects is set to one)
;-----------------------------------------------------------------------------------------
    if ply_usesoundeffects

ply_sfx_track3_pitch 
    ld   de,0
    exx
ply_sfx_track3_volume   equ $+2
ply_sfx_track3_note     equ $+1
    ld   de,0                   ; d=inverted volume e=note
ply_sfx_track3_instrument 
    ld   hl,0                   ; if 0, no sound effect.
    ld   a,l
    or   h
    jr   z,ply_sfx_track3_end
    ld   a,1
    ld   (ply_ps_endsound_sfx + 1),a
    call ply_playsound
    xor  a
    ld   (ply_ps_endsound_sfx + 1),a
    ld   a,l                    ; if the new address is 0, the instrument is over. speed is set in the process, we don't care.
    or   h
    jr   z,ply_sfx_track3_instrument_setaddress

ply_sfx_track3_instrumentspeedcpt 
    ld   a,1
    dec  a
    jr   nz,ply_sfx_track3_playnoforward
ply_sfx_track3_instrument_setaddress
    ld   (ply_sfx_track3_instrument + 1),hl
ply_sfx_track3_instrumentspeed 
    ld   a,6
ply_sfx_track3_playnoforward
    ld   (ply_sfx_track3_instrumentspeedcpt + 1),a

ply_sfx_track3_end

    endif
;------------------------------------------

	
	; ---------------------------------------------------------------------------
;*** end of arkos tracker player
