.module _cpc_DrawFastLine
.globl _cpc_DrawFastLine

;; mode 1
;; a x x x a x x x
;; x b x x x b x x
;; x x c x x x c x
;; x x x d x x x d

;; 0,1 = x1
;; 2,3 = y1
;; 4,5 = x2
;; 6,7 = y2
;; 8 = colour

.area _CODE
_cpc_DrawFastLine:
push ix
ld ix,#4
add ix,sp


;; pixel into byte
ld a,0(ix)
and #0x1
ld (pixel),a
ld c,#0xaa
jr z,cdfl1
ld c,#0x55
cdfl1:
;; full colour pen
ld a,8(ix)
;; masked for putting onto screen
and c
;; store in colour
ld (cur_colour),a

;; mask, invert so we can isolate pixels when doing replace method
ld a,c
cpl
ld (cur_mask),a


;; dx>dy

ld e,0(ix)		;; x1
ld d,1(ix)
srl d			;; /2 because 2 pixels per byte
rr e
ld l,2(ix)		;; y1
ld h,3(ix)
call _cpc_GetScrAddrInternal
ld (scraddr),hl

ld l,4(ix)    	;; x2
ld h,5(ix)
ld e,0(ix)  	;; x1
ld d,1(ix)
or a
sbc hl,de   ;; dx = x2-x1



;; restart colour is always left pixel
ld a,8(ix)
and #0x0aa
ld (res_colour),a

;; restart mask is always right pixel
ld a,#0x055
ld (res_mask),a

xor a
ld (left_right_flag),a

bit 7,h
jr z,not_neg_dx


;; restart colour is always left pixel
ld a,8(ix)
and #0x055
ld (res_colour),a

;; restart mask is always right pixel
ld a,#0x0aa
ld (res_mask),a


ld a,#1
ld (left_right_flag),a

;; negate dy to make it +ve
ld a,l
cpl
ld l,a
ld a,h
cpl
ld h,a
inc a

not_neg_dx:
ld (dx),hl

add hl,hl		;; dx = 2*dx
ld (dx2),hl

xor a
ld (up_down_flag),a

ld l,6(ix)		;; y2
ld h,7(ix)
ld e,2(ix)		;; y1
ld d,3(ix)
or a
sbc hl,de 		;; dy = y2 - y1
bit 7,h
jr z,not_neg_dy


ld a,#1
ld (up_down_flag),a

;; negate dy to make it +ve
ld a,l
cpl
ld l,a
ld a,h
cpl
ld h,a
inc a

not_neg_dy:
ld (dy),hl
add hl,hl
ld (dy2),hl		;; HL = 2*dy


ld hl,(dy2)
ld de,(dx)
or a
sbc hl,de		;; d = 2*dy - dx
ld (_d),hl

ld hl,(dy2) 
ld de,(dx2)
or a
sbc hl,de		;; ((2*dy)-(2*dx))
ld (dy2subdx2),hl

;; initial pixel
call plot_pixel
;; update x and counter
jr upd_x

more_pixels:
ld a,b
or c
jr z,drawLineDone

ld hl,(_d)
bit 7,h
jr nz,neg_or_zero
ld a,h
or l
jr z,neg_or_zero

call up_down_line

call plot_pixel

ld hl,(_d)
ld de,(dy2subdx2)
add hl,de
ld (_d),hl
jr upd_x

drawLineDone:
pop ix
ret


neg_or_zero:

call plot_pixel

ld hl,(_d)
ld de,(dy2)
add hl,de
ld (_d),hl



upd_x:
;; works out pixel and stuff
ld a,(pixel)
inc a
and #0x1
ld (pixel),a
or a
jr nz,more_pixels2

;; reset colour/mask
ld a,(res_colour)
ld (cur_colour),a
ld a,(res_mask)
ld (cur_mask),a

call  left_right_pixel_do

ld bc,(dx)
dec bc
ld (dx),bc

jr more_pixels

up_down_line:
ld hl,(scraddr)
call up_down_line_do
ld (scraddr),hl
ret

up_down_line_do:
ld a,(up_down_flag)
or a
jp  nz,_cpc_PrevLineInternal
jp _cpc_NextLineInternal 

left_right_pixel_do2:
ld a,(left_right_flag)
or a
jr nz,lrpd2r

;; shift...
ld a,(cur_colour)
rrca
ld (cur_colour),a

;; shift really
ld a,(cur_mask)
rrca
ld (cur_mask),a
ret
lrpd2r:
ld a,(cur_colour)
rlca
ld (cur_colour),a

;; shift really
ld a,(cur_mask)
rlca
ld (cur_mask),a
ret


left_right_pixel_do:
ld a,(left_right_flag)
or a
jr z,move_right
jr move_left

move_right:
ld hl,(scraddr)
inc hl
ld (scraddr),hl
ret

move_left:
ld hl,(scraddr)
dec hl
ld (scraddr),hl
ret

more_pixels2:
call left_right_pixel_do2

ld bc,(dx)
dec bc
ld (dx),bc

jp more_pixels

plot_pixel:
ld hl,(scraddr)
ld a,(cur_colour)
ld c,a
call plot_pixel_replace
ret
  

;; C = colour
plot_pixel_replace:
ld a,(cur_mask)
and (hl)
or c
ld (hl),a
ret

;; C = colour
plot_pixel_xor:
ld a,(hl)
xor c
ld (hl),a
ret




.area _DATA
pixel:
.db 0
res_colour:
.db 0
res_mask:
.db 0
up_down_flag:
.db 0
left_right_flag:
.db 0
cur_colour:
.db 0
cur_mask:
.db 0
dy2subdx2:
.dw 0
dy2:
.dw 0
dy:
.dw 0
_d:
.dw 0
dx:
.dw 0
dx2:
.dw 0
scraddr:
.dw 0
