;; ---------------------------------------------------------------------------
;; Supergrafx hardware definition / (c) SyX 2013
;; ---------------------------------------------------------------------------

;; ---------------------------------------------------------------------------
;; CTC
;; ---------------------------------------------------------------------------
;; I/O Ports
CTC_TIM0            = 0xF880   ;; Channel 0 (I: 4 MHz clock | O: YMZ clock)
CTC_TIM1            = 0xF881   ;; Channel 1 (I: Cursor CRTC | O: NMI)
CTC_TIM2            = 0xF882   ;; Channel 2 (I: 4 MHz clock | O: CTC_TIM3 TRG)
CTC_TIM3            = 0xF883   ;; Channel 3 (I: CTC_TIM2 TO)

;; ---------------------------------------------------------------------------
;; D0-D7 : Data bus (R/W) Used for communicating with the Z80
;; ---------------------------------------------------------------------------
;; Control Word
;; ------------
;; D7: Interrupt (0: Disable / 1: Enable)
;; D6: Mode (0: Timer / 1: Counter)
;; D5: Prescaler Value (0: 16 / 1: 256) *** Only in Timer mode
;; D4: Clock/Trigger Edge Selection (0: Falling Edge / 1: Rising Edge)
;; D3: Timer Trigger (0: Automatic Trigger when time constant is loaded 
;;                    1: CLK/TRG pulse starts timer) *** Only in Timer mode
;; D2: Time Constant (0: No Time Constant / 1: Time Constant follows)
;; D1: Reset (0: Continued Operation / 1: Software Reset)
;; D0: Control or Vector (0: Vector / 1: Control Word)
;; ---------------------------------------------------------------------------
;; Constants for Control Word
CTC_NO_INT          = $%00000000
CTC_INT             = $%10000000
CTC_USE_TIMER       = $%00000000
CTC_USE_COUNTER     = $%01000000
CTC_PRESCALER_16    = $%00000000
CTC_PRESCALER_256   = $%00100000
CTC_FALLING_EDGE    = $%00000000
CTC_RISING_EDGE     = $%00010000
CTC_AUTO_TRIGGER    = $%00000000
CTC_CLK_TRIGGER     = $%00001000
CTC_NO_CONSTANT     = $%00000000
CTC_CONSTANT        = $%00000100
CTC_NO_RESET        = $%00000000
CTC_RESET           = $%00000010
CTC_VECTOR          = $%00000000
CTC_CONTROL         = $%00000001

;; Typical values
CTC_START_TIMER16   = CTC_NO_INT|CTC_USE_TIMER|CTC_PRESCALER_16|CTC_RISING_EDGE|CTC_AUTO_TRIGGER|CTC_CONSTANT|CTC_RESET|CTC_CONTROL
CTC_START_TIMER256  = CTC_NO_INT|CTC_USE_TIMER|CTC_PRESCALER_256|CTC_RISING_EDGE|CTC_AUTO_TRIGGER|CTC_CONSTANT|CTC_RESET|CTC_CONTROL
CTC_START_ITIMER16  = CTC_INT|CTC_USE_TIMER|CTC_PRESCALER_16|CTC_RISING_EDGE|CTC_AUTO_TRIGGER|CTC_CONSTANT|CTC_RESET|CTC_CONTROL
CTC_START_ITIMER256 = CTC_INT|CTC_USE_TIMER|CTC_PRESCALER_256|CTC_RISING_EDGE|CTC_AUTO_TRIGGER|CTC_CONSTANT|CTC_RESET|CTC_CONTROL
CTC_START_COUNTER   = CTC_NO_INT|CTC_USE_COUNTER|CTC_PRESCALER_256|CTC_RISING_EDGE|CTC_AUTO_TRIGGER|CTC_CONSTANT|CTC_RESET|CTC_CONTROL
CTC_START_ICOUNTER  = CTC_INT|CTC_USE_COUNTER|CTC_PRESCALER_256|CTC_RISING_EDGE|CTC_AUTO_TRIGGER|CTC_CONSTANT|CTC_RESET|CTC_CONTROL
CTC_STOP_CHANNEL    = CTC_RESET|CTC_CONTROL
    
;; ---------------------------------------------------------------------------
;; YMZ
;; ---------------------------------------------------------------------------
;; I/O Ports
YMZ_SELECT_LEFT     = 0xF984   ;; Select register (Left YMZ)
YMZ_WRITE_LEFT      = 0xF884   ;; Write in the selected register (Left YMZ)
YMZ_SELECT_RIGHT    = 0xF988   ;; Select register (Right YMZ)
YMZ_WRITE_RIGHT     = 0xF888   ;; Write in the selected register (Right YMZ)

;; ---------------------------------------------------------------------------
;; YMZ Registers
YMZ_REG_00          = 0x00     ;; Channel A Fr=ency (8 Bit Fine Tune)
YMZ_REG_01          = 0x01     ;; Channel A Fr=ency (4 Bit Coarse Tune)
YMZ_REG_02          = 0x02     ;; Channel B Fr=ency (8 Bit Fine Tune)
YMZ_REG_03          = 0x03     ;; Channel B Fr=ency (4 Bit Coarse Tune)
YMZ_REG_04          = 0x04     ;; Channel C Fr=ency (8 Bit Fine Tune)
YMZ_REG_05          = 0x05     ;; Channel C Fr=ency (4 Bit Coarse Tune)
YMZ_REG_06          = 0x06     ;; Noise Fr=ency (5 Bit Period Control)
YMZ_REG_07          = 0x07     ;; Mixer Setting (Noise CBA + Tone CBA) 
YMZ_REG_08          = 0x08     ;; Channel A Volume (Envelope + 4 Bits)
YMZ_REG_09          = 0x09     ;; Channel B Volume (Envelope + 4 Bits)
YMZ_REG_0A          = 0x0A     ;; Channel c Volume (Envelope + 4 Bits)
YMZ_REG_0B          = 0x0B     ;; Envelope Fr=ency (8 Bit Fine Tune)
YMZ_REG_0C          = 0x0C     ;; Envelope Fr=ency (8 Bit Coarse Tune)
YMZ_REG_0D          = 0x0D     ;; Envelope Shape (4 Bits)
YMZ_REG_0F          = 0x0F     ;; Control Power ($%0000----)

;; ---------------------------------------------------------------------------
